% PointMap demo on the Circle-in-Square data set
% (c) Norbert Kopco 2002

clear all
figure;

% create the training set
Iv=rand(10000,2);
Ov= (sum((abs(Iv-0.5)).^2,2)>1/(2*pi))+1;
ind1=find(Ov==1); ind2=find(Ov==2);
subplot(2,2,1); plot(Iv(ind1,1),Iv(ind1,2),'r.',Iv(ind2,1),Iv(ind2,2),'b.');
axis square
title('Training set: 10000 random points from two classes (IN or OUT the circle)');

% training
disp('Training starts!');
%input('Hit ENTER to start training!');
gamma = 0.25; % weigh equally criticallity and predictive accuracy
Cmax = 1000;   % do not use more than 50 nodes for this problem
[w,o, delta]=pointmaptrain(Iv, Ov, gamma, Cmax);
disp('Training done!');

% create testing set
NUM_TPATS=10000;
steps = round(sqrt(NUM_TPATS))+1; ax = mod(0:1/steps:steps,1); ax = ax/max(max(ax)); 
ay=floor(0:1/steps:steps)/steps; ay = ay/max(max(ay(1:end-1))); a = [ax(1:end-1); ay(1:end-1)]; clear ax ay;
I2v=a'; clear ax ay a;
O2v= (sum((abs(I2v-0.5)).^2,2)>1/(2*pi))+1;
ind1=find(O2v==1); ind2=find(O2v==2);
subplot(2,2,2); plot(I2v(ind1,1),I2v(ind1,2),'r.',I2v(ind2,1),I2v(ind2,2),'b.');
axis square
title('Testing set: 10201 uniformly distributed points');

% testing
k=1; % durind testing consider only the nearest neighbor
disp('Testing w/ no post-training pruning starts!');
O3=pointmaptest( w, o, I2v, 1);
ind1=find(O3==2); 
subplot(2,2,3); l1=plot(I2v(ind1,1),I2v(ind1,2),'.','color',[.8 .8 .8]);
axis square
hold on
plot(w(:,1),w(:,2),'b.');
title(sprintf('Results: Training w/ C_{max}=%d and gamma=%5.2f.\nCoding nodes retained after training (blue) and the approximated decision boundary\n(border between gray and white). No post-training pruning. Accuracy: %5.2f%%',Cmax,gamma,sum(O3==O2v)/length(O2v)*100));
disp('Testing w/ no post-training pruning done!');

% testing w/ post-training pruning
theta = 0.75; % post-training pruning keeps 75% of nodes
disp('Post-training pruning starts!');
[wpruned, opruned] = pointmappostprune( w, o, delta, theta);
disp('Post-training pruning done!');
disp('Testing starts!');
O4=pointmaptest( wpruned, opruned, I2v, 1);
disp('Testing done!');
ind1=find(O4==2); 
subplot(2,2,4); l1=plot(I2v(ind1,1),I2v(ind1,2),'.','color',[.8 .8 .8]);
axis square
hold on
plot(wpruned(:,1),wpruned(:,2),'b.');
title(sprintf('Results: Same simulation as on left but with post-training \npruning that keeps %5.1f%% of nodes. Accuracy: %5.2f%%',theta*100,sum(O4==O2v)/length(O2v)*100));
